// constants
// aircraft line
var LAT=0;
var LON=1;
var MODES=2;
var ALT=3;
var ICON=4;
var COORD=5;
var REGISTRATION=6;
var FLIGHTNUMBER=7;
var ROUTE=8;
var TYPE=9;
var SPEED=10;
var HEADING=11;
var VERTICALVELOCITY=12;
var COUNTRY=13;
// options line
var COLOUR=0;
var WIDTH=1;
var ONLINE=0;
var UPDATE=1;
var LABELS=2;
var SHOWCON=3;
var CONTIME=4;
var CONTRANS=5;
var SHOWBOXBG=6;
var FGCOLOUR=7;
var BGCOLOUR=8;
var LABELTRANS=9;
var TEXTSIZE=10;
var TRANSALT=11;
var SHOWFRAME=12;
var LABELLAYOUT=13;
// routes line
var ICAOCODE=0;
var ICAOAIRPORTNAME=1;

var ASCII_A=65;

// initialize XMLHttpRequest object
var xmlobj=null;
var xmlhttpExists=null;

var readCount=1;
var dataFile = '~FlightData.xml';

var timeoutID = 0;
var flightTimeoutTime = 20000; 	// 20s before flights time out
var xmlFileTimeoutTime = 10000;	// 10s before xml read times out
var mapMarkers = [];		// plane icon markers
var flights = [];			// positions of flight numbers
var updatedFlights = [];		// flight flagged as updated
var markerNo = 0;
var maxMarkers = 0;
var numToDelete = 0;
var labelOrder="";
var transAlt=5000;
var timeToNextUpdate = 0;

var ICAOCodes = [];
var ICAOAiportNames = [];

// GEPlugin stuff
var GEpluginModeS="";
var GElastheading = -1;
var roll = 0.0;
var rollSustain=0;
var GEUpdateCount=0;
var camera = null; 
var GELaunchtime=0;
var oldTimeAwareDoc =0;

var ownMarker=new Array();			// own markers
var ownMarkersAdded = 0;

var polylines=new Array();			// plane contrails
var polyRecent=new Array();		// most recent point in contrail

var ticksNow = 0;
var timeIssued = 0;
var timeGot = 0;
var commsRetries = 0;
var consecutivecommsRetries = 0;
var requestTimer = 0;
var lastUpdate = 0;
var fso = null;
var xmlDoc = null;

if (runLocal)
{
   fso=new ActiveXObject("Scripting.FileSystemObject");
   xmlDoc=new ActiveXObject("Microsoft.XMLDOM"); 
}

// send http request
function sendRequest(doc)
{
   // check for existing requests
   if (xmlobj != null && xmlobj.readyState != 0 && xmlobj.readyState != 4)
   {
      xmlobj.abort();
   }
   try
   {
      // instantiate object for Mozilla, Nestcape, etc.
      if (xmlobj == null)
      {
         xmlobj=new XMLHttpRequest();
         xmlhttpExists=new XMLHttpRequest();
      }
   }
   catch(e)
   {
      try
      {
         // instantiate object for Internet Explorer
         if (xmlobj == null)
         {
            xmlobj=new ActiveXObject('Microsoft.XMLHTTP');
            xmlhttpExists=new ActiveXObject('Microsoft.XMLHTTP');
         }
      }
      catch(e)
      {
         // Ajax is not supported by the browser
         xmlobj=null;
         return false;
      }
   }

   var date = new Date();
   ticksNow = date.getTime();

   if (runLocal)
   {
      xmlDoc.async="false";
      xmlDoc.load(doc); 
      processXML(xmlDoc);
   }
   else
   {
      // assign state handler
      xmlobj.onreadystatechange=stateChecker;

      // open socket connection
      timeIssued = date.getTime();
      xmlobj.open('GET', doc+"?rnd="+ticksNow, true);

      requestTimer = setTimeout(function() { retryComms(); }, xmlFileTimeoutTime);

      // send GET request
      xmlobj.send(null);
   }
}


// check request status
function stateChecker()
{
   // if request is completed
   if (xmlobj.readyState==4)
   {
      clearTimeout(requestTimer);

      // if status == 200 display text file
      if (xmlobj.status==200)
      {
         if (xmlobj.responseXML != null)
         {
            consecutivecommsRetries = 0;
            var date = new Date();
            timeGot = date.getTime();
            processXML(xmlobj.responseXML);
         }
         else
         {
            // xmlobj.responseXML = null
            retryComms();
         }
      }
      else
      {
//         alert('Failed to get response :'+ xmlobj.statusText);
         retryComms();
      }
   }
}


function retryComms()
{
   xmlobj.abort(); commsRetries++; consecutivecommsRetries++; updateTitleBar(); 

   if (consecutivecommsRetries < 10)
   {
      setTimeout("sendRequest(dataFile)",updateFrequencyInSecs * 1000);
   }
   else
   {
      alert('10 consecutive communications failures, stopping. Status: ' + xmlobj.statusText);
   }
}


function processXML(xmlDoc)
{
   // read XML data
   var optionsData = xmlDoc.getElementsByTagName('o');
   var routeData = xmlDoc.getElementsByTagName('r');
   var aircraftData = xmlDoc.getElementsByTagName('a');
   var contrailData = xmlDoc.getElementsByTagName('l');

   if (optionsData != null)
   {
      processOptions(optionsData);
   }

   if (routeData != null)
   {
      processRoutes(routeData);
   }

   if (aircraftData != null)
   {
      // display XML data
      displayData(aircraftData, contrailData);
  }
}


function processOptions(optionsData)
{
   var optionsDataLength = 0;

   try
   {
      optionsDataLength = optionsData.length;
   }
   catch (e)
   {
      // no data available, skip this update
   }

   if (optionsDataLength == 0)
   {
      return;
   }

   var oData = optionsData[0].getAttribute("O").split("/");

   if (oData[ONLINE] != 1)
   {
      window.location.reload();
      return;
   }

   if (oData[UPDATE] <= lastUpdate)
   {
      return;
   }

   lastUpdate = oData[UPDATE];

   Lbls = parseInt(oData[LABELS]);
   Cont = parseInt(oData[SHOWCON]);
   ContTim = parseInt(oData[CONTIME]);
   var newContTrans = parseInt(oData[CONTRANS]);
   var newLblHasBg = parseInt(oData[SHOWBOXBG]);
   lblTrns = 100 - parseInt(oData[LABELTRANS]);
   newTxtSize = parseInt(oData[TEXTSIZE]);
   newBdr = parseInt(oData[SHOWFRAME]);
   labelOrder = oData[LABELLAYOUT];
   transAlt = parseFloat(oData[TRANSALT]);

   if (newContTrans != ConTrans)
   {
      ConTrans = newContTrans;
      removeAllContrails();
   } 

   Contransp = ConTrans / 100.0;

   if (newTxtSize != TxtSize)
   {
      TxtSize = newTxtSize;
      changecss('.aircraftlabel','font-size',TxtSize+'%');
   }

   if (newLblHasBg != LblHasBg)
   {
      LblHasBg = newLblHasBg;
      changecss('.aircraftlabel','background-color','transparent');
   }
   else
   {
      var newLblFg = oData[FGCOLOUR];
      var newLblBg = oData[BGCOLOUR];

      if (newLblFg != LblFg)
      {
         LblFg = newLblFg;
         changecss('.aircraftlabel','color',LblFg);
      }

      if (newLblBg != LblBg)
      {
         LblBg = newLblBg;
         changecss('.aircraftlabel','background-color',LblBg);
      }
   }

   if (newBdr != Bdr)
   {
      Bdr = newBdr;
      bdrSetting = 'black 1px ';

      if (Bdr == 0)
      {
         bdrSetting += 'hidden';
      }
      else
      {
         bdrSetting += 'solid';
      }
      changecss('.aircraftlabel','border',bdrSetting);
   }
}


function processRoutes(routeData)
{
   var routeDataLength = 0;

   try
   {
      routeDataLength = routeData.length;
   }
   catch (e)
   {
      // no data available, skip this update
      return;
   }

   if (routeDataLength == 0)
   {
      return;
   }

   for (var i = 0; i < routeDataLength; i++)
   {
      var rData = routeData[i].getAttribute("a").split("/");
      var nextIndex = ICAOCodes.length;
      var gotCode = 0;

      for (var j = 0; j < nextIndex; j++)
      {
         if (ICAOCodes[j] == rData[ICAOCODE])
         {
            gotCode = 1;
            break;
         }
      }

      if (gotCode == 0)
      {
         ICAOCodes[nextIndex] = rData[ICAOCODE];
         ICAOAiportNames[nextIndex] = rData[ICAOAIRPORTNAME];
      }
   }
}



function updateTitleBar()
{
   if (showDebugInfo == 0)
   {
      return;
   }

   var date = new Date();

   var hours = date.getHours();
   var mins = date.getMinutes();
   var secs = date.getSeconds();

   if (hours < 10)
   {
      hours = "0" + hours;
   }
   if (mins < 10)
   {
      mins = "0" + mins;
   }
   if (secs < 10)
   {
      secs = "0" + secs;
   }

   document.title = hours + ":" + mins + ":" + secs + "   Loop:" + readCount++;

   var ticks = timeGot - timeIssued;
   document.title += " [" + ticks + "]";

   if (commsRetries > 0)
   {
      document.title += " Retries: " + commsRetries;
   }
}


// display data at a given time interval
function displayData(aircraftData, contrailData)
{
   if (aircraftData == null)
   {
      //alert("null: timeToNextUpdate = " + timeToNextUpdate);
      if (runLocal)
      {
         aircraftData = xmlDoc.getElementsByTagName('a');
         contrailData = xmlDoc.getElementsByTagName('l');
      }
      else
      {
         aircraftData = xmlobj.responseXML.getElementsByTagName('a');
         contrailData = xmlobj.responseXML.getElementsByTagName('l');
      }
   }

   updateTitleBar();

   var aircraftDataLength = 0;
   var contrailDataLength = 0;

   try
   {
      aircraftDataLength = aircraftData.length;
   }
   catch (e)
   {
      // no data available, skip this update
   }

   try
   {
      contrailDataLength = contrailData.length;
   }
   catch (e)
   {
      // no data available, skip this update
   }

   if (runLocal)
   {
      if (numToDelete > 0)
      {
         setTimeout("showFlights();",250);
         return;
      }

      //==== reset flights if options changed
      if (fso.FileExists(resetflightsLocation))
      {
         ResetFlights(); 
         setTimeout("showFlights();",100);
         return;
      }

      //==== reload page if requested (update freq change)
      if (fso.FileExists(reloadPageLocation))
      {
	 fso.DeleteFile(reloadPageLocation,true);
         window.location.reload();
         return;
      }
   }


   //==== add own markers from OwnMarkers.xml file
   if (ownMarkersAdded == 0 && FileExists(ownMarkersHTML))
   {
      AddOwnMarkers();
   }

   var numFlights = 0;

   //=============  remove any flights that have timed out
   for(var j=0; j < updatedFlights.length; j++)
   {
      if (updatedFlights[j] > 0)
      {
         if (ticksNow - updatedFlights[j] >= flightTimeoutTime)
         {
	    removeAircraft(j);
         }
         else
         {
            numFlights++;
         }
      }
   }


   //============= Update the page title
   if (showDebugInfo == 0)
   {
      document.title = progName + " -";

      if (numFlights > 0)
      {
         document.title += " " + numFlights + " flight";

         if (numFlights != 1)
         {
            document.title += "s";
         }  

         document.title += " from";
      }

      document.title += " " + clientName;
   }

   //=============  show the flights read from the xml file
   for (var i=0; i < aircraftDataLength; i++)
   {
      var aData = aircraftData[i].getAttribute("p").split("/");

      var imageURL = "MapImages/"+aData[ICON]+".png";
      var coord = parseInt(aData[COORD]);

      var decLat = parseFloat(aData[LAT]);
      var decLon = parseFloat(aData[LON]);
      var decAlt = parseFloat(aData[ALT]);
      var decCountry = parseInt(aData[COUNTRY]);

      var decHeading = 0.0;
      if (aData[HEADING] != "-")
      {
         decHeading = parseFloat(aData[HEADING]);
      }
      var decSpeed = 0.0;
      if (aData[SPEED] != "-")
      {
         decSpeed = parseFloat(aData[SPEED]);
      }
      var decVerticalVelocity = 0.0;
      if (aData[VERTICALVELOCITY] != "-")
      {
         decVerticalVelocity = parseFloat(aData[VERTICALVELOCITY]);
      }

      if (timeToNextUpdate > 0)
      {
         for (var markNum=0; markNum < maxMarkers; markNum++)
         {
            if (flights[markNum] == aData[MODES])
            {
               // not an xml update yet, use interpolated position
               var pos = mapMarkers[markNum].position();
               decLat = pos.lat();
               decLon = pos.lng();
               decAlt = mapMarkers[markNum].zindex();
               aData[LAT] = decLat.toFixed(4); // was toString();
               aData[LON] = decLon.toFixed(4); // was toString();
               aData[ALT] = decAlt.toString();
            }
         }
      }
      else
      {
         // xml update, calculate positions in relation to estimated positions
         for (var markNum=0; markNum < maxMarkers; markNum++)
         {
            if (mapMarkers[markNum] != 0)
            {
               if (flights[markNum] == aData[MODES] && mapMarkers[markNum].speed() > 0)
               {
                  // got new position, measure angle and distance from last position
                  var pos = mapMarkers[markNum].position();
                  var bearing = bearingFromLatLon(pos.lat(), pos.lng(), decLat, decLon);
                  var distance = distanceFromLatLon(pos.lat(), pos.lng(), decLat, decLon);
                  var angleDiff = Math.abs(differenceBetweenAngles(bearing, mapMarkers[markNum].heading()));

                  if (angleDiff > 60.0)
                  {
                     // would move backwards, use current position
                     decLat = pos.lat();
                     decLon = pos.lng();
                     aData[LAT] = decLat.toString();
                     aData[LON] = decLon.toString();
                  }

                  if (angleDiff > 90.0)
                  {
                     distance = -distance;
                  }

                  if (Math.abs(distance) > 1.0)
                  {
                     if (distance < 0)
                     {
                        if (mapMarkers[markNum].throttle() > 0.2)
                        {
                           mapMarkers[markNum].setThrottle(mapMarkers[markNum].throttle() - 0.2);
                        }
                     }
                     else
                     {
                        if (mapMarkers[markNum].throttle() < 1.5)
                        {
                           mapMarkers[markNum].setThrottle(mapMarkers[markNum].throttle() + 0.2);
                        }
                     }
                  }

                  if (Math.abs(mapMarkers[markNum].verticalVelocity()) > 5.0)
                  {
                     // correct incorrect altitudes
                     var altDiff = decAlt - mapMarkers[markNum].zindex();
                     if ((altDiff < 0 && mapMarkers[markNum].verticalVelocity() > 0.0) || 
                         (altDiff > 0 && mapMarkers[markNum].verticalVelocity() < 0.0))
                     {
                        // would move in wrong direction, use current alt
                        decAlt = mapMarkers[markNum].zindex();
                        aData[ALT] = decAlt.toString();
                     }                    
                  }                    
               }
            }
         }
      }

      // correct decimal separator if not '.'
      if (decimalSeparator != ".")
      {
         aData[LAT] = aData[LAT].replace(".", decimalSeparator);
         aData[LON] = aData[LON].replace(".", decimalSeparator);
         aData[ALT] = aData[ALT].replace(".", decimalSeparator);
         aData[HEADING] = aData[HEADING].replace(".", decimalSeparator);
         aData[SPEED] = aData[SPEED].replace(".", decimalSeparator);
      }

      // get the aircraft photo
      var photo = "NoPhoto.png";
      if (aData[REGISTRATION] != "-")
      {
         photo = "";
         if (photoLocation.length > 0)
         {
            if (runLocal)
            {
               if (fso.FileExists(photoLocation + aData[REGISTRATION] + ".jpg"))
               {
                  photo = photoLocation + aData[REGISTRATION] + ".jpg";
               }
            }
         }

         if (photo.length == 0)
         {
            photo = "http://www.airnavsystems.com/cgi-bin/ANLV_SV/Photo/" + aData[REGISTRATION] + ".jpg";
         }
      }

      var detailsHTML="";
      var decAltToNearest10 = Math.round(decAlt / 10.0) * 10;


      if (Lbls == CLICKTOVIEW)
      {
         detailsHTML='<center><div class="detailslabel">' +
           '<table cellpadding=0 cellspacing=0>' +
           '<tr bgcolor="#00A0FF"><td align=center width=242><font color="#FFFFFF">Aircraft details</font></td><td align=right><img hspace=3 src="Close.png" onClick="closeClickToViewBox()"></td></tr>' +
           '</table>' +
           '<table cellpadding=0 width=240 class="detailstext">' +
           '<tr><td width=110>Flight: ' + aData[FLIGHTNUMBER] + '</td><td align=left width=130>ModeS: ' + aData[MODES] + '</td></tr>' +
           '<tr><td>Reg.: ' + aData[REGISTRATION] + '</td><td>Type: ' + aData[TYPE] + '</td></tr>' +
           '<tr><td>Altitude: ' + decAltToNearest10 + 'ft</td><td>Speed: ' + aData[SPEED];

         if (aData[SPEED] != '-')
         {
            detailsHTML+= 'kts';
         }

         detailsHTML+= '</td></tr>' + '<tr><td>Heading: ' + aData[HEADING];

         if (aData[HEADING] != '-')
         {
            detailsHTML+= '&deg';
         }

         detailsHTML+= '</td><td>Vert.Vel.: ' + aData[VERTICALVELOCITY];

         if (aData[VERTICALVELOCITY] != '-')
         {
            detailsHTML+= 'ft/min';
         }

         var fullRoute = "-";
         if (aData[ROUTE] != '-')
         {
            fullRoute = expandRoute(aData[ROUTE]);
         }

         var airlineID = "-";
         if (aData[FLIGHTNUMBER] != '(')
         {
            airlineID = airline( ( ((aData[FLIGHTNUMBER].charCodeAt(0) - ASCII_A) * 26 +
                                    (aData[FLIGHTNUMBER].charCodeAt(1) - ASCII_A)) * 26 ) + 
	                            (aData[FLIGHTNUMBER].charCodeAt(2) - ASCII_A) );
         }

         detailsHTML+= '</td></tr>' +
           '<tr><td>Latitude: ' + aData[LAT] + '</td><td>Longitude: ' + aData[LON] + '</td></tr>' +
           '<tr><td colspan=2>Airline: ' + airlineID + '</td></tr>' +
           '<tr><td colspan=2>Country: ' + countryFromCode(decCountry) + '</td></tr>' +
           '<tr><td colspan=2>Route: ' + fullRoute + '</td></tr>' +
           '<tr><td align=center colspan=2><img vspace=10 src=\"' + photo + '\"/></td></tr>';

         if (GEPluginOp == 0 && aData[SPEED] != '-' && aData[HEADING] != '-' && aData[VERTICALVELOCITY] != '-')
         {
            detailsHTML += '<tr><td align=center colspan=2><button style="width:110;height:24" ' + 
                'onClick="ShowGEPlugin(\'' + aData[MODES] + '\',' + decLat + ',' + decLon + ')">' +
                '<b>Cockpit view</b></button></td></tr>'; 
         }

         detailsHTML+= '</table></center>';
      }

      labelHTML='<div class="aircraftlabel">';
      for (var code = 0; code < labelOrder.length; code++)
      {
         switch(labelOrder.charAt(code))
         {
            case 'M':
               labelHTML += "(" + aData[MODES] + ") ";
               break;
            case 'H':
               if (aData[HEADING] != "-")
               {
                  labelHTML += aData[HEADING] + "&deg ";
               }
               break;
            case 'L':
               if (decAlt >= transAlt)
               {
                  labelHTML += "FL" + Math.round(decAlt / 100) + " ";
               }
               else
               {
                  labelHTML += decAltToNearest10 + "ft ";
               }
               break;
            case 'S':
               if (aData[SPEED] != "-")
               {
                  labelHTML += aData[SPEED] + "kts ";
               }
               break;
            case 'T':
               if (aData[TYPE].length > 0)
               {
                  labelHTML += "(" + aData[TYPE] + ") ";
               }
               break;
            case 'R':
               if (aData[REGISTRATION] != '-')
               {
                  labelHTML += aData[REGISTRATION] + " ";
               }
               break;
            case 'P':
               if (aData[ROUTE] != '-')
               {
                  labelHTML += aData[ROUTE] + " ";
               }
               break;
            case 'V':
               if (aData[VERTICALVELOCITY] != "-")
               {
                  labelHTML += aData[VERTICALVELOCITY] + "ft/min ";
               }
               break;
            case 'F':
               if (aData[FLIGHTNUMBER] != "-")
               {
                  labelHTML += aData[FLIGHTNUMBER] + " ";
               }
               else
               {
                  labelHTML += "(" + aData[MODES] + ") ";
               }
               break;
            case ';':
               labelHTML = labelHTML.trim(' ');
               labelHTML += "<BR>";
               break;
         }
      }

      // find the line that corresponds to this marker
      var j=0;
      while (j < contrailDataLength)
      {
         var lineID=contrailData[j].getAttribute("c").split("/");

         if (lineID[MODES] == aData[MODES])
         {
            break;
         }

         j++;
      }

      // see if this flight already has a marker/line
      for (markerNo = 0; markerNo < flights.length; markerNo++)
      {
         if (flights[markerNo] == aData[MODES])
         { 
            var offsets = new getIconOffsets(imageURL);
            mapMarkers[markerNo].setOffsets(offsets.xOffset, offsets.yOffset);
            var newLatLng = new google.maps.LatLng(decLat,decLon);
            mapMarkers[markerNo].setPosition(markerNo,newLatLng,decAlt,decHeading,decSpeed,
                      decVerticalVelocity,labelHTML,detailsHTML);

            if (imageURL != mapMarkers[markerNo].getIcon())
            {
               mapMarkers[markerNo].setIcon(imageURL);
            }

            break;
         }
      }

      // if flight was not found then try and re-use an empty space
      if (markerNo >= flights.length)
      {
         for (markerNo = 0; markerNo < flights.length; markerNo++)
         {
            if (flights[markerNo] == "")
            {
               break;
            }
         }

         // add the new aircraft marker and its label
         flights[markerNo] = aData[MODES];

         if (markerNo >= maxMarkers)
         {
            lb(markerNo,decLat,decLon,decAlt,labelHTML,detailsHTML,imageURL);
            polylines[markerNo] = 0;
            maxMarkers++;
         }
         else
         {
            // re-use an existing marker
            var offsets = new getIconOffsets(imageURL);
            mapMarkers[markerNo].setOffsets(offsets.xOffset, offsets.yOffset);
            var newLatLng = new google.maps.LatLng(decLat,decLon);
            mapMarkers[markerNo].setPosition(markerNo,newLatLng,decAlt,decHeading, decSpeed,
                                 decVerticalVelocity,labelHTML,detailsHTML);
            mapMarkers[markerNo].setIcon(imageURL);
            mapMarkers[markerNo].show();
            polyRecent[markerNo] = 0;

            if (polylines[markerNo] != 0)
            {
               var clearPts = [];
               polylines[markerNo].setPath(clearPts);
               polylines[markerNo].setOptions({strokeOpacity: 1.0-Contransp});
            }
         }
      }

      updatedFlights[markerNo] = ticksNow;	// save the time of update

      if (aData[MODES] == GEpluginModeS)
      {
         UpdateGEPlugin(decLat,decLon,decAlt,decHeading,decSpeed,decVerticalVelocity);
      }

      // if aircraft not moved, skip further updates
      if (interpolate == 0 && coord <= polyRecent[markerNo])
      {
         continue;    // not moved, no contrails to redraw
      }

      //=========== handle polylines (aircraft contrails)
      if (j < contrailDataLength && Cont != 0)
      {
         // read each point on that line
         var points=contrailData[j].getAttribute("p").split("/");
         var pts = [];
         var firstLat = decLat;
         var firstLon = decLon;

         for (var k = 0; k < points.length; k++)
         {
            if (k == 0)
            {
               pts[k] = new google.maps.LatLng(decLat,decLon);
            }
            else
            {
               var pData = points[k].split(";");
               var lineLat = firstLat + (parseFloat(pData[LAT]) / 10000.0);
               var lineLon = firstLon + (parseFloat(pData[LON]) / 10000.0);
               pts[k] = new google.maps.LatLng(lineLat,lineLon);
            }
         }

         if (polylines[markerNo] == 0)
         {
            polylines[markerNo] = new google.maps.Polyline({  
                     path: pts,
                     strokeColor: lineID[COLOUR],
                     strokeOpacity: 1.0-Contransp,
                     strokeWeight: lineID[WIDTH]  });

            polylines[markerNo].setMap(map);
         }
         else	// else, updating an existing polyline
         {
            if (coord > polyRecent[markerNo] || timeToNextUpdate > 0)	// ANDY FIX added timeToNextUpdate > 0
            {
               polylines[markerNo].setPath(pts);
            }
         }

         polyRecent[markerNo] = coord;
      }


   }

   if (interpolate != 0)
   {
      if (timeToNextUpdate <= 0)
      {
         setTimeout("sendRequest(dataFile)",10);
         timeToNextUpdate = updateFrequencyInSecs * 1000;
      }
      else
      {
         timeToNextUpdate -= 500;
         interpolatePositions();
         setTimeout("displayData();",500);
      }
   }
   else
   {
      setTimeout("sendRequest(dataFile)", updateFrequencyInSecs * 1000);
   }
}


function showFlights()
{
    // check if browser is DOM compatible
    if(document.getElementById && document.getElementsByTagName && document.createElement)
    {
        // load XML file
        if (runLocal)
        {
           dataFile = localFlightDataLocation;
        }

        sendRequest(dataFile);
    }
}


function interpolatePositions()
{
   for (var markNum=0; markNum < maxMarkers; markNum++)
   {
      if (flights[markNum] != "" && 
             mapMarkers[markNum].heading() >= 0 && 
             mapMarkers[markNum].speed() >= 0 &&
             mapMarkers[markNum].zindex() > 300)
      {
         var knotsToMetresPerSec = 0.514444444 / 2.0;  // = 2 updates per sec
         var distance = (mapMarkers[markNum].speed() * mapMarkers[markNum].throttle() * knotsToMetresPerSec) + "m";
         var pos = mapMarkers[markNum].position();

         var newLatLon = Point_At_Distance_And_Bearing(pos.lat(), pos.lng(), distance, mapMarkers[markNum].heading().toString());

         var altitudeInFt = mapMarkers[markNum].zindex();

         if (mapMarkers[markNum].verticalVelocity() != 0)
         {
            // convert from ft/min to ft per half sec
            var feetDiff = 0.40 * mapMarkers[markNum].verticalVelocity() / (60.0 * 2.0);
            altitudeInFt = altitudeInFt + feetDiff;
         }

         newLatLon[0] = newLatLon[0].toFixed(4);
         newLatLon[1] = newLatLon[1].toFixed(4);
         var newLat = parseFloat(newLatLon[0]);
         var newLon = parseFloat(newLatLon[1]);
         var newLatLng = new google.maps.LatLng(newLat,newLon);
         mapMarkers[markNum].updatePosition(markNum,newLatLng,altitudeInFt);

         if (flights[markNum] == GEpluginModeS)
         {
            UpdateGEPlugin(newLat,newLon, mapMarkers[markNum].zindex(), mapMarkers[markNum].heading(),
                mapMarkers[markNum].speed(), mapMarkers[markNum].verticalVelocity() );
         }
      }
   }
}


function AddOwnMarkers()
{
   if (runLocal == 1)
   {
       xmlDoc.async="false";
       xmlDoc.load(ownMarkersHTML); 
//       xmlObj=xmlDoc.documentElement;	// ANDY FIX not needed
       ProcessXMLMarkerDoc(xmlDoc);
   }
   else
   {
      // assign state handler
      xmlobj.onreadystatechange=ownMarkerStateChecker;

      // open socket connection
      xmlobj.open('GET',ownMarkersHTML,true);

      // send GET request
      xmlobj.send(null);
   }
}


function ownMarkerStateChecker()
{
   // if request is completed
   if (xmlobj.readyState==4)
   {
      // if status == 200 display text file
      if (xmlobj.status==200)
      {
         if (xmlobj.responseXML != null)
         {
            ProcessXMLMarkerDoc(xmlobj.responseXML);
         }
         else
         {
            // xmlobj.responseXML = null
            return;
         }
      }
      else
      {
         alert('Failed to get response :'+ xmlobj.statusText);
      }
   }
}


function ProcessXMLMarkerDoc(xmlData)
{
 var ownmarkers = 0;

   try
   {
      ownmarkers = xmlData.getElementsByTagName("marker");
   }
   catch(e)
   {
      alert("ERROR adding own markers");
      return;
   }

   for (var i = 0; i < ownmarkers.length; i++)
   {
      var name = ownmarkers[i].getAttribute("name");
      var lat=parseFloat(toDecDeg(ownmarkers[i].getAttribute("lat")));
      var lon=parseFloat(toDecDeg(ownmarkers[i].getAttribute("lon")));
      var type = ownmarkers[i].getAttribute("type");
      var width = parseInt(ownmarkers[i].getAttribute("width"));
      var height = parseInt(ownmarkers[i].getAttribute("height"));
      var xoffset = -parseInt(ownmarkers[i].getAttribute("xoffset"));
      var yoffset = -parseInt(ownmarkers[i].getAttribute("yoffset"));

      var labelHTML="";

      if (name.length > 0)
      {
        labelHTML='<div class="ownmarkerlabel">'+name+'</div>';
      }

      om(ownMarkersAdded++,lat,lon,width,height,xoffset,yoffset,labelHTML,type);
   } 
}

function ResetFlights()
{ // local function only
   if (runLocal)
   {
      if (fso.FileExists(resetflightsLocation))
      {
         fso.DeleteFile(resetflightsLocation,true);
      }
   }

   removeAllContrails();
   polylines = [];
   polyRecent = [];

   for (var j = 0; j < maxMarkers; j++)
   {
      if (mapMarkers[j] != 0)
      {  
         mapMarkers[j].setIcon(null);
         mapMarkers[j].setMap(null);
         mapMarkers[j] = 0;

         updatedFlights[j] = 0;
         flights[j] = "";
         numToDelete++;
      }
   }

   maxMarkers = markerNo = 0;
   mapMarkers = [];
   flights = [];
   updatedFlights = [];

   for(var j = 0; j < ownMarkersAdded; j++)
   {
      ownMarker[j].setMap(null);
   }
   ownMarkersAdded = 0;
   ownMarker = [];
}


function removeAircraft(index)
{
   mapMarkers[index].hide();

   if (polylines[index] != 0)
   {
/*
      var clearPts = [];
      polylines[index].setPath(clearPts);
      polylines[index].setOptions({strokeOpacity: 1.0-Contransp});
*/
      var clearPts = [];
      polylines[index].setPath(clearPts);
      polylines[index].setMap(null);
      polylines[index] = 0;


   }

   polyRecent[index] = 0;
   updatedFlights[index] = 0;
   flights[index] = "";
}


function removeAllContrails()
{
   for(var j = 0; j < maxMarkers; j++)
   {
      if (polylines[j] != 0)
      {
         polylines[j].setMap(null);
         polylines[j] = 0;
      }
   }

   polylines = [];
   polyRecent = [];
}

function ShowGEPlugin(modeS,lat,lng)
{
   GEpluginModeS = modeS;

   var mapBase = document.getElementById('mapbase');

   mapBase.innerHTML = '<center><div class="detailslabel">' +
      '<table cellpadding=0 cellspacing=0>' +
      '<tr bgcolor="#00A0FF"><td align=center width=' + CockpitWidth + '><font color="#FFFFFF">Cockpit View</font></td>' +
      '<td align=right><img hspace=3 src="Close.png" onClick="closeCockpitView()"></td></tr>' +
         '</table><table height=' + GEPluginHeight + 'px><tr><td>&nbsp;</td></tr></table></center>';
   mapBase.style.visibility = "visible";

   var map3D = document.getElementById('map3d');
   map3D.style.visibility="visible"; 

   roll = 0.0;
   rollSustain=0;
   GElastheading = -1;
   GEUpdateCount=0;

   ge.getOptions().setFlyToSpeed(4.0); 

   if (showDaylightCtrl != 0)
   {
      GELaunchtime=ticksNow;
      ge.getSun().setVisibility(false);
      ge.getSun().setVisibility(true);
   }

   if (camera == null)
   {
      camera = ge.getView().copyAsCamera(ge.ALTITUDE_RELATIVE_TO_GROUND); 
   }
   camera.setLatitude(lat); 
   camera.setLongitude(lng);

   ge.getView().setAbstractView(camera);

   if (showRealTimeClouds != 0)
   {
      var kmlString = "<?xml version='1.0' encoding='UTF-8'?>" +
             "<kml xmlns='http://earth.google.com/kml/2.1'>" +
             "<NetworkLink >" +
             "<visibility>1</visibility>" + 
             "<open>1</open>" + 
             "<Link><href>http://www.barnabu.co.uk/files/kmz/global-cloud-layer.kmz</href></Link>" +
             "<flyToView>0</flyToView>" +
             "</NetworkLink>" +
             "</kml>";

      ge.getFeatures().appendChild(ge.parseKml(kmlString));
   }
}

function UpdateGEPlugin(lat,lng,alt,heading,speed,verticalVelocity)
{
   if (++GEUpdateCount > 4)
   {
      ge.getOptions().setFlyToSpeed(0.5); 
   }

   camera.setLatitude(lat);
   camera.setLongitude(lng);
   camera.setAltitude(alt * 0.3048); // convert feet to metres

   var angle = 0.0;
   if (Math.abs(verticalVelocity) > 0.001)
   {
      angle = getAngleOfDescent(speed,verticalVelocity);
   }

   camera.setTilt(90.0 + angle + GEHorizonTilt);
   camera.setHeading(heading); 

   if (GElastheading >= 0 && Math.abs(heading - GElastheading) >= 1.0)
   {
      var angleDiff = differenceBetweenAngles(heading, GElastheading);

      var rollAmount = 20.0;

      if (Math.abs(angleDiff) < 5.0)
      {
         rollAmount = 5.0;
      }
      else
      {
         if (Math.abs(angleDiff) < 10.0)
         {
            rollAmount = 10.0;
         }
      }

      if (Math.abs(rollAmount) > 10)
      {
         rollSustain = 40;
      }

      if (angleDiff < 0)
      {
         roll = -rollAmount;
      }
      else
      {
         roll = rollAmount;
      }
   }

   if (rollSustain != 0)
   {
      rollSustain--;
   }
   else
   {
      if (roll != 0)
      {
         if (roll > 0)
         {
            roll -= 1;
         }
         else
         {
            roll += 1;
         }
      }
   }

   camera.setRoll(roll);

   GElastheading = heading;

   if (showDaylightCtrl != 0)
   {
      var date = new Date();

/*
      var timeStrg = '2010-02-26T' + PadDigits(date.getHours(),2) + ':' + PadDigits(date.getMinutes(),2) + ':' +
				 PadDigits(date.getSeconds(),2) + 'Z';

      var kmlString = '<?xml version="1.0" encoding="UTF-8"?>' +
        '<kml xmlns="http://www.opengis.net/kml/2.2"' + 
        ' xmlns:gx="http://www.google.com/kml/ext/2.2">' +
        '  <Document>' +
        '      <Camera>' +
        '        <gx:TimeStamp>' +
        '           <when>' + timeStrg + '</when>' +
        '        </gx:TimeStamp>' +
        '      </Camera>' +
        '      <TimeSpan>' +
        '        <begin>' + timeStrg + '</begin>' +
        '        <end>' + timeStrg + '</end>' +
        '      </TimeSpan>' +
        '  </Document>' +
        '</kml>';

      var timeAwareDoc = ge.parseKml(kmlString);

      if (oldTimeAwareDoc)
      {
         ge.getFeatures().replaceChild(timeAwareDoc, oldTimeAwareDoc);
      }
      else
      {
         ge.getFeatures().appendChild(timeAwareDoc);
      }

      oldTimeAwareDoc = timeAwareDoc;

      if (ticksNow - GELaunchtime > 60000)
      {
         ge.getSun().setVisibility(false);
         ge.getSun().setVisibility(true);
         ge.getSun().setVisibility(false);
         ge.getSun().setVisibility(true);
         GELaunchtime=ticksNow;
      }
*/
   }

   ge.getView().setAbstractView(camera);
}


function getAngleOfDescent(speed,verticalVelocity)
{
   var angleTan = Math.abs(verticalVelocity) /
			 (speed * 101.2685916); // = knots to feet moved in 1 minute = 5280 * (1.15077945 / 60)
   var angleRetn = rad2deg( Math.atan(angleTan) );

   if (verticalVelocity < 0.0)
   {
      angleRetn = -angleRetn;
   }

   return angleRetn;
}


function GEPluginAircraftChange(index)
{
   // new aircraft clicked on while GE Plugin visible
   if (GEpluginModeS != "")
   {
      GEpluginModeS = flights[index];
      roll = 0.0;
      rollSustain=0;
      GElastheading = -1;
      GEUpdateCount=0;

      if (mapMarkers[index].heading() >= 0)
      {
         ge.getOptions().setFlyToSpeed(4.0); 
         camera.setTilt(0.0); 
         camera.setRoll(0.0); 
      }
      else
      {
         GEpluginModeS = "";
      }
   }
}


function closeCockpitView()
{
   ge.getSun().setVisibility(false);
   document.getElementById('mapbase').style.visibility="hidden"; 
   document.getElementById('map3d').style.visibility="hidden"; 
   GEpluginModeS="";
   roll = 0.0;
   GElastheading = -1;
}


function FileExists(strURL)
{
    if (runLocal)
    {
       return fso.FileExists(strURL);
    }
    else
    {
       xmlhttpExists.open("HEAD", strURL, false);
       xmlhttpExists.send();
       return (xmlhttpExists.status==404) ? false : true;
    }
}

String.prototype.trim = function()
{
   return this.replace(/^\s+|\s+$/g,"");
}


function differenceBetweenAngles(firstAngle, secondAngle)
{
   var difference = secondAngle - firstAngle;
   while (difference < -180) difference += 360;
   while (difference > 180) difference -= 360;
   return difference;
}

function toDecDeg(latlon)
{
   lastChar = latlon.charAt(latlon.length-1);
   latlon = latlon.replace(",",".");

   if (lastChar == 'N' || lastChar == 'S' || lastChar == 'E' || lastChar == 'W')
   {
      var length=latlon.length;
      var decVal = parseFloat(latlon.substr(0,length-5)) +
          (parseFloat(latlon.substr(length-5,2)) + 
          (parseFloat(latlon.substr(length-3,2)) / 60.0)) / 60.0; 

      if (lastChar == 'S' || lastChar == 'W')
      {
         decVal=-decVal;
      }

      var strg=decVal.toString();
      return strg.replace(",",".");
   }

return latlon;
}


function PadDigits(n, totalDigits) 
{ 
   n=n.toString(); 
   var pd = ''; 
   if (totalDigits > n.length) 
   { 
      for (i=0; i < (totalDigits-n.length); i++) 
      { 
         pd += '0'; 
      } 
   } 
   return pd + n.toString(); 
}  


function expandRoute(route)
{
   var airportsVisited = route.split("-");

   var returnedStrg = "";

   for (var i = 0; i < airportsVisited.length; i++)
   {
      for (var j = 0; j < ICAOCodes.length; j++)
      {
         if (ICAOCodes[j] == airportsVisited[i])
         {
            returnedStrg += ICAOAiportNames[j] + " ";
            break;
         }
      }

      returnedStrg += "(" + airportsVisited[i] +  ")";

      if (i < airportsVisited.length-1)
      {
         if (i < airportsVisited.length-2)
         {
            returnedStrg += " via ";
         }
         else
         {
            returnedStrg += " to ";
         }
      }

   }

   return returnedStrg;
}

function countryFromCode(countryCode)
{
  switch(countryCode)
  {
  case 0:
     return "(unallocated 1)";
  case 1:
     return "Zimbabwe";
  case 2:
     return "Mozambique";
  case 3:
     return "South Africa";
  case 4:
     return "Egypt";
  case 5:
     return "Libya";
  case 6:
     return "Morocco";
  case 7:
     return "Tunisia";
  case 8:
     return "Botswana";
  case 9:
     return "Burundi";
  case 10:
     return "Cameroon";
  case 11:
     return "Comoros";
  case 12:
     return "Congo";
  case 13:
     return "Cote d Ivoire";
  case 14:
     return "Gabon";
  case 15:
     return "Ethiopia";
  case 16:
     return "Equatorial Guinea";
  case 17:
     return "Ghana";
  case 18:
     return "Guinea";
  case 19:
     return "Guinea-Bissau";
  case 20:
     return "Lesotho";
  case 21:
     return "Kenya";
  case 22:
     return "Liberia";
  case 23:
     return "Madagascar";
  case 24:
     return "Malawi";
  case 25:
     return "Maldives";
  case 26:
     return "Mali";
  case 27:
     return "Mauritania";
  case 28:
     return "Mauritius";
  case 29:
     return "Niger";
  case 30:
     return "Nigeria";
  case 31:
     return "Uganda";
  case 32:
     return "Qatar";
  case 33:
     return "Central African Republic";
  case 34:
     return "Rwanda";
  case 35:
     return "Senegal";
  case 36:
     return "Seychelles";
  case 37:
     return "Sierra Leone";
  case 38:
     return "Somalia";
  case 39:
     return "Swaziland";
  case 40:
     return "Sudan";
  case 41:
     return "Tanzania";
  case 42:
     return "Chad";
  case 43:
     return "Togo";
  case 44:
     return "Zambia";
  case 45:
     return "Democratic Republic of";
  case 46:
     return "Angola";
  case 47:
     return "Benin";
  case 48:
     return "Cape Verde";
  case 49:
     return "Djibouti";
  case 50:
     return "Djibouti";
  case 51:
     return "Gambia";
  case 52:
     return "Burkina Faso";
  case 53:
     return "Sao Tome";
  case 54:
     return "Algeria";
  case 55:
     return "Bahamas";
  case 56:
     return "Barbados";
  case 57:
     return "Belize";
  case 58:
     return "Colombia";
  case 59:
     return "Costa Rica";
  case 60:
     return "Cuba";
  case 61:
     return "El Salvador";
  case 62:
     return "Guatemala";
  case 63:
     return "Guyana";
  case 64:
     return "Haiti";
  case 65:
     return "Honduras";
  case 66:
     return "Saint Vincent and the Grenadines";
  case 67:
     return "Jamaica";
  case 68:
     return "Nicaragua";
  case 69:
     return "Panama";
  case 70:
     return "Dominican Republic";
  case 71:
     return "Trinidad and Tobago";
  case 72:
     return "Suriname";
  case 73:
     return "Antigua and Barbuda";
  case 74:
     return "Grenada";
  case 75:
     return "Mexico";
  case 76:
     return "Venezuela";
  case 77:
     return "Russian Federation";
  case 78:
     return "(reserved for AFI)";
  case 79:
     return "Namibia";
  case 80:
     return "Eritrea";
  case 81:
     return "(reserved for SAM)";
  case 82:
     return "Italy";
  case 83:
     return "Spain";
  case 84:
     return "France";
  case 85:
     return "Germany";
  case 86:
     return "United Kingdom";
  case 87:
     return "Austria";
  case 88:
     return "Belgium";
  case 89:
     return "Bulgaria";
  case 90:
     return "Denmark";
  case 91:
     return "Finland";
  case 92:
     return "Greece";
  case 93:
     return "Hungary";
  case 94:
     return "Norway";
  case 95:
     return "Netherlands";
  case 96:
     return "Poland";
  case 97:
     return "Portugal";
  case 98:
     return "Czech Republic";
  case 99:
     return "Romania";
  case 100:
     return "Sweden";
  case 101:
     return "Switzerland";
  case 102:
     return "Turkey";
  case 103:
     return "Yugoslavia";
  case 104:
     return "Cyprus";
  case 105:
     return "Ireland";
  case 106:
     return "Iceland";
  case 107:
     return "Luxembourg";
  case 108:
     return "Malta";
  case 109:
     return "Monaco";
  case 110:
     return "San Marino";
  case 111:
     return "(reserved for EUR/NAT)";
  case 112:
     return "Albania";
  case 113:
     return "Croatia";
  case 114:
     return "Latvia";
  case 115:
     return "Lithuania";
  case 116:
     return "Republic of Moldova";
  case 117:
     return "Slovakia";
  case 118:
     return "Slovenia";
  case 119:
     return "Uzbekistan";
  case 120:
     return "Ukraine";
  case 121:
     return "Belarus";
  case 122:
     return "Estonia";
  case 123:
     return "Georgia";
  case 124:
     return "Tajikistan";
  case 125:
     return "Macedonia (FYROM)";
  case 126:
     return "Bosnia and Herzegovina";
  case 127:
     return "Armenia";
  case 128:
     return "(reserved for MID)";
  case 129:
     return "Azerbaijan";
  case 130:
     return "Kyrgyzstan";
  case 131:
     return "Turkmenistan";
  case 132:
     return "(reserved for ASIA)";
  case 133:
     return "Bhutan";
  case 134:
     return "Micronesia";
  case 135:
     return "Mongolia";
  case 136:
     return "Kazakhstan";
  case 137:
     return "Palau";
  case 138:
     return "Afghanistan";
  case 139:
     return "Bangladesh";
  case 140:
     return "Myanmar";
  case 141:
     return "Kuwait";
  case 142:
     return "Lao People's Democratic";
  case 143:
     return "Nepal";
  case 144:
     return "Oman";
  case 145:
     return "Cambodia";
  case 146:
     return "Saudi Arabia";
  case 147:
     return "Korea";
  case 148:
     return "Democratic People's Rep";
  case 149:
     return "Iraq";
  case 150:
     return "Iran";
  case 151:
     return "Israel";
  case 152:
     return "Jordan";
  case 153:
     return "Lebanon";
  case 154:
     return "Malaysia";
  case 155:
     return "Philippines";
  case 156:
     return "Pakistan";
  case 157:
     return "Singapore";
  case 158:
     return "Sri Lanka";
  case 159:
     return "Syria";
  case 160:
     return "China";
  case 161:
     return "Australia";
  case 162:
     return "India";
  case 163:
     return "Japan";
  case 164:
     return "Thailand";
  case 165:
     return "Vietnam";
  case 166:
     return "Yemen";
  case 167:
     return "Bahrain";
  case 168:
     return "Brunei Darussalam";
  case 169:
     return "United Arab Emirates";
  case 170:
     return "Solomon Islands";
  case 171:
     return "Papua New Guinea";
  case 172:
     return "Taiwan";
  case 173:
     return "Indonesia";
  case 174:
     return "(reserved for NAM/PAC)";
  case 175:
     return "Marshall Islands";
  case 176:
     return "Cook Islands";
  case 177:
     return "Samoa";
  case 178:
     return "United States";
  case 179:
     return "(reserved for future use)";
  case 180:
     return "Canada";
  case 181:
     return "New Zealand";
  case 182:
     return "Fiji";
  case 183:
     return "Nauru";
  case 184:
     return "Saint Lucia";
  case 185:
     return "Tonga";
  case 186:
     return "Kiribati";
  case 187:
     return "Vanuatu";
  case 188:
     return "(reserved for future use)";
  case 189:
     return "Argentina";
  case 190:
     return "Brazil";
  case 191:
     return "Chile";
  case 192:
     return "Ecuador";
  case 193:
     return "Paraguay";
  case 194:
     return "Peru";
  case 195:
     return "Uruguay";
  case 196:
     return "Bolivia";
  case 197:
     return "(reserved for CAR)";
  case 198:
     return "ICAO (1)";
  default:
     return "Unknown";
  }
}


function changecss(theClass,element,value)
{
   var cssRules;
   var added = false;

   for (var S = 0; S < document.styleSheets.length; S++)
   {
      if (document.styleSheets[S]['rules'])
      {
         cssRules = 'rules';
      }
      else if (document.styleSheets[S]['cssRules'])
      {
         cssRules = 'cssRules';
      }
      else
      {
         //no rules found... browser unknown
      }

      for (var R = 0; R < document.styleSheets[S][cssRules].length && !added; R++)
      {
         if (document.styleSheets[S][cssRules][R].selectorText == theClass)
         {
	    if(document.styleSheets[S][cssRules][R].style[element])
            {
	       document.styleSheets[S][cssRules][R].style[element] = value;
               added=true;
               break;
            }
         }
      }

      if(!added)
      {
         if(document.styleSheets[S].insertRule)
         {
            document.styleSheets[S].insertRule(theClass+' { '+element+': '+value+'; }',document.styleSheets[S][cssRules].length);
         }
         else if (document.styleSheets[S].addRule)
         {
	    document.styleSheets[S].addRule(theClass,element+': '+value+';');
         }
      }
   }
}
	